package gap.enekow.dlappse;

import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothSocket;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.SharedPreferences;
import android.os.Handler;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.Toast;

import java.io.IOException;
import java.lang.reflect.Method;
import java.util.UUID;

/**
 * Created by enekow on 30/05/2015.
 */
public class ConnectionManager extends Thread {

    String TAG = "GAKOA";

    private BluetoothAdapter btAdapter;
    private BluetoothDevice dollyDevice;
    private BluetoothSocket mmSocket;
    private Context ctx;
    private Handler mHandler;
    private IntentFilter filter;
    private boolean dollyWasFound = false;
    public static Connection connection;
    public static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB");

    public ConnectionManager(Context context, Handler handler) {
        filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        ctx = context;
        mHandler = handler;
        btAdapter = BluetoothAdapter.getDefaultAdapter();

        configureReceivers();

        if (isBluetoothReady()){
            btAdapter.startDiscovery();
        }else{
            Log.e(TAG, "Bluetootha ez dago prest");
        }
    }

    /**
     * BrodcastReceiver-a konfiguratu gure filtroekin
     */
    private void configureReceivers(){
        ctx.registerReceiver(receiver, filter);
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_STARTED);
        ctx.registerReceiver(receiver, filter);
        filter = new IntentFilter(BluetoothAdapter.ACTION_DISCOVERY_FINISHED);
        ctx.registerReceiver(receiver, filter);
        filter = new IntentFilter(BluetoothAdapter.ACTION_STATE_CHANGED);
        ctx.registerReceiver(receiver, filter);
        filter = new IntentFilter(BluetoothDevice.ACTION_BOND_STATE_CHANGED);
        ctx.registerReceiver(receiver, filter);
        filter = new IntentFilter(BluetoothDevice.ACTION_ACL_DISCONNECTED);
        ctx.registerReceiver(receiver, filter);
    }

    /**
     * Bluetooth-aren egoera egiaztatu prest dagoen ikusteko
     */
    public boolean isBluetoothReady(){
        //Android dispositiboak bluetooth antena duen egiaztatu
        if (btAdapter == null) {
            Toast.makeText(ctx, ctx.getString(R.string.toast_bt_detection_failed), Toast.LENGTH_SHORT).show();
            Log.e(TAG, "Ez da Bluetooth antena detektatu");
            return false;
        }
        //Bluetooth-a aktibatuta dagoen egiaztatu
        if (!btAdapter.isEnabled()) {
            Toast.makeText(ctx, ctx.getString(R.string.toast_bt_must_be_activated), Toast.LENGTH_LONG).show();
            Log.e(TAG, "Bluetootha ez dago aktibatua");
            return false;
        }
        return true;
    }

    /**
     * BroadcastReceiver-ak Android gailuaren aldaketak detektatzen ditu (Observer)
     */
    private final BroadcastReceiver receiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            int prevBondState = 0;

            switch (action) {
                case BluetoothAdapter.ACTION_DISCOVERY_STARTED:
                    Log.v(TAG, "Bt bilaketa hasi");
                    break;
                case BluetoothDevice.ACTION_FOUND:
                    //Aurkitutako gailuarentzat lekua erreserbatu
                    BluetoothDevice device = intent.getParcelableExtra(BluetoothDevice.EXTRA_DEVICE);
                    //Dolly-a den egiaztatu
                    if (isOurDolly(device)) {
                        dollyDevice = device;
                        Log.v(TAG, "Dolly-a aurkitu da");
                        dollyWasFound = true;
                    }
                    if(dollyWasFound){
                        //Aurkitutako gailua parekatuta dagoen egiaztatu
                        if (dollyDevice.getBondState() == BluetoothDevice.BOND_BONDED) {
                            Log.v(TAG, "Dolly-a parekatua dago");
                            Log.v(TAG, "Konektatzen saiatzen...");
                            createSocket(dollyDevice);
                        } else {
                            //Parekatzeko eskaera egin
                            Log.v(TAG, "Dolly-a ez dago parekatua");
                            pairDevice(dollyDevice);
                        }
                    }
                    break;
                /* Discovery amaitzean */
                case BluetoothAdapter.ACTION_DISCOVERY_FINISHED:
                    if (!dollyWasFound) {
                        Toast.makeText(context, ctx.getString(R.string.toast_dolly_not_found), Toast.LENGTH_LONG).show();
                    }
                    break;
                /* Bluetooth-a desaktibatu bada */
                case BluetoothAdapter.ACTION_STATE_CHANGED:
                    if (btAdapter.getState() == btAdapter.STATE_OFF) {
                        Log.e(TAG, "Blueooth-a desaktibatu egin da");
                    }
                    break;
                /* Bluetooth parekatzeko saiakera */
                case BluetoothDevice.ACTION_BOND_STATE_CHANGED:
                    if (dollyDevice.getBondState() == BluetoothDevice.BOND_BONDING) {
                        //Parekatzeko saiakera
                        Log.v(TAG, "Dispositiboak parekatzen");
                        Toast.makeText(context, ctx.getString(R.string.toast_password), Toast.LENGTH_LONG).show();
                    } else {
                        if (prevBondState == BluetoothDevice.BOND_BONDING && dollyDevice.getBondState() == BluetoothDevice.BOND_BONDED) {
                            Log.v(TAG, "Dispositiboak parekatu dira");
                            Toast.makeText(context, ctx.getString(R.string.toast_dolly_pair_succes), Toast.LENGTH_LONG).show();
                        } else {
                            Log.e(TAG, "Dispositiboak ezin izan dira parekatu");
                            Toast.makeText(context, ctx.getString(R.string.toast_dolly_pair_fail), Toast.LENGTH_LONG).show();
                        }
                    }
                    break;
                /* Bluetooth konexioa galtzen bada */
                case BluetoothDevice.ACTION_ACL_DISCONNECTED:
                    dollyWasFound = false;
                    mHandler.obtainMessage(Constants.CONNECTION_LOST).sendToTarget();
                    break;
            }
        }
    };//BroadcastReceiver()

    private void createSocket(BluetoothDevice device){
        // Use a temporary object that is later assigned to mmSocket,
        // because mmSocket is final
        BluetoothSocket tmp = null;
        //mmDevice = device;
        // Get a BluetoothSocket to connect with the given BluetoothDevice
        try {
            // MY_UUID is the app's UUID string, also used by the server code
            tmp = device.createRfcommSocketToServiceRecord(MY_UUID);
        } catch (IOException e) { }
        mmSocket = tmp;
        runSocket();
    }


    private boolean isOurDolly(BluetoothDevice device){
        //Bluetooth moduluaren-aren izena lortu konfigurazio fitxategitik
        SharedPreferences sharedPrefs = PreferenceManager.getDefaultSharedPreferences(ctx);
        String dollyName = sharedPrefs.getString(ctx.getString(R.string.pref_dollyName_key), ctx.getString(R.string.pref_dollyName_defaultName));
        if (device.getName().equals(dollyName)){
            return true;
        }else{
            return false;
        }
    }

    private void pairDevice(BluetoothDevice device) {
        try {
            Log.d(TAG, "Parekatzen saiatzen");
            Method m = device.getClass()
                    .getMethod("createBond", (Class[]) null);
            m.invoke(device, (Object[]) null);
        } catch (Exception e) {
            Log.e(TAG, e.getMessage());
        }
    }

    public void runSocket() {
        // Discovery-a deskonektatu, dagoeneko aurkitu dugu Arduinoa eta konexioa mantsotuko luke
        btAdapter.cancelDiscovery();

        try {
            // Socketaren bitartez konektatu
            mmSocket.connect();
        } catch (IOException connectException) {
            // Ezin da konektatu, konexioa itxi eta irten
            Log.e(TAG, "Ezin izan da konektatu");
            try {
                Log.e(TAG, "Socket-a ixten saiatzen");
                mmSocket.close();
            } catch (IOException closeException) { }
            return;
        }

        // Konexioa kudeatu beste metodo batean
        manageConnectedSocket(mmSocket);
    }

    /** Irekitako socketarekin konektatu eta abisuak pasatu UI hariari */
    private void manageConnectedSocket(BluetoothSocket socket){
        connection = new Connection(socket, mHandler);
        Log.v(TAG, "Konexioa ezarri da");

        mHandler.obtainMessage(Constants.SUCCESS_CONNECT).sendToTarget();

        //Entzuteko beste hari bat sortu
        Thread listenThread = new Thread(new Connection(socket, mHandler));
        listenThread.start();
    }

    /** Abian dagoen konexioa ezabatzen du */
    public void cancel() {
        Log.v(TAG, "Konexioa ezeztatzen");
        try {
            mmSocket.close();
        } catch (IOException e) { }
    }
}