package gap.enekow.dlappse;

import android.app.AlertDialog;
import android.bluetooth.BluetoothAdapter;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.os.Message;
import android.preference.PreferenceManager;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentPagerAdapter;
import android.support.v4.app.FragmentTransaction;
import android.support.v4.view.ViewPager;
import android.support.v7.app.ActionBar;
import android.support.v7.app.ActionBarActivity;
import android.util.DisplayMetrics;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.Toast;

import java.util.Locale;

public class MainActivity extends ActionBarActivity
        implements ActionBar.TabListener, TimelapseFragment.TimelapseFragmentListener, ManualFragment.ManualFragmentListener{

    /**
     * The {@link android.support.v4.view.PagerAdapter} that will provide
     * fragments for each of the sections. We use a
     * {@link FragmentPagerAdapter} derivative, which will keep every
     * loaded fragment in memory. If this becomes too memory intensive, it
     * may be best to switch to a
     * {@link android.support.v4.app.FragmentStatePagerAdapter}.
     */
    SectionsPagerAdapter mSectionsPagerAdapter;

    /**
     * The {@link ViewPager} that will host the section contents.
     */
    ViewPager mViewPager;

    String TAG = "GAKOA";

    private BluetoothAdapter btAdapter;
    private Context ctx;
    private Connection connection;
    private int state;
    private Locale myLocale;
    private String prevLang;

    boolean shouldExecuteOnResume;
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        /** Layout-a ezarri */
        setContentView(R.layout.activity_main);
        shouldExecuteOnResume = false;
        /** Aldagaien hasieraketak eta hasierako konfigurazioak */
        //filter = new IntentFilter(BluetoothDevice.ACTION_FOUND);
        btAdapter = BluetoothAdapter.getDefaultAdapter();
        ctx = getApplicationContext();
        state=Constants.STATUS_NOT_READY;
        setupHandler();

        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(ctx);
        String lang = preferences.getString(ctx.getString(R.string.pref_language_key), "en");
        prevLang=lang;

        myLocale = new Locale(lang);
        Resources res = getResources();
        Configuration conf = res.getConfiguration();
        DisplayMetrics dm = res.getDisplayMetrics();
        conf.locale = myLocale;
        res.updateConfiguration(conf, dm);

        /** ActionBar konfigurazioa */
        final ActionBar actionBar = getSupportActionBar();
        actionBar.setNavigationMode(ActionBar.NAVIGATION_MODE_TABS);
        // Adapter bat sortu fragment bat itzuliko duena
        mSectionsPagerAdapter = new SectionsPagerAdapter(getSupportFragmentManager());
        // Adapter-aren sekzioekin ViewPager-a hasieratu
        mViewPager = (ViewPager) findViewById(R.id.pager);
        mViewPager.setAdapter(mSectionsPagerAdapter);

        // Sekzio batetik bestera igarotzean, aukeratu dagokion Tab-a
        mViewPager.setOnPageChangeListener(new ViewPager.SimpleOnPageChangeListener() {
            @Override
            public void onPageSelected(int position) {
                actionBar.setSelectedNavigationItem(position);
            }
        });

        // Aplikazioaren sekzio bakoitzeko Tab bat gehitu ActionBar-ean
        for (int i = 0; i < mSectionsPagerAdapter.getCount(); i++) {
            actionBar.addTab(
                    actionBar.newTab()
                            .setText(mSectionsPagerAdapter.getPageTitle(i))
                            .setTabListener(this));
        }

        //Connection Manager haria abiarazi
        Thread th = new Thread(new ConnectionManager(ctx, mHandler));
        th.start();
    }

    /**
     *
     * @param cmd Bidali behar den komandoaren identifikadorea
     * @param data Komandoak behar duen informazioa
     */
    public void sendCommand(int cmd, String data){
        switch (cmd){
            case Constants.STOP:
                if(state != Constants.STATUS_TIMELAPSE_IN_PROGRESS){
                    connection.write("STOP\n");
                    state=Constants.STATUS_READY;
                    setStartStop_Start();
                }else{
                    showTimelapseStopDialog();
                }
                break;
            case Constants.GO_LEFT:
                state=Constants.STATUS_GOING_LEFT;
                //setStartStop_Stop();
                connection.write("LEFT" + data +"\n");
                break;
            case Constants.GO_RIGHT:
                state=Constants.STATUS_GOING_RIGHT;
                //setStartStop_Stop();
                connection.write("RIGH" + data +"\n");
                break;
            case Constants.TAKE_A_SHOT:
                connection.write("SHOT\n");
                break;
            case Constants.START_STOP_TIMELAPSE:
                switch (state){
                    case Constants.STATUS_TIMELAPSE_IN_PROGRESS:
                        showTimelapseStopDialog();
                        break;
                    case Constants.STATUS_GOING_LEFT:
                        connection.write("STOP\n");
                        setStartStop_Start();
                        state=Constants.STATUS_READY;
                        break;
                    case Constants.STATUS_GOING_RIGHT:
                        connection.write("STOP\n");
                        setStartStop_Start();
                        state=Constants.STATUS_READY;
                        break;
                    case Constants.STATUS_READY:
                        startTimeLapse(data);
                        break;
                    default:
                        break;
                }
                break;
            case Constants.SEND_CONFIG:
                sendConfig();
                break;
            default:
                break;
        }
    }

    /**
     * Handler-a
     * UIaren eta harien artean komunikazioa ahalbidetzeko
     */
    private Handler mHandler;
    private void setupHandler(){
        mHandler = new Handler(Looper.getMainLooper()) {
            @Override
            public void handleMessage(Message inputMessage) {
                switch(inputMessage.what){
                    case Constants.SUCCESS_CONNECT:
                        connection = ConnectionManager.connection;
                        Toast.makeText(getApplicationContext(), ctx.getString(R.string.toast_connection_success), Toast.LENGTH_SHORT).show();
                        sendCommand(Constants.SEND_CONFIG, null);
                        shouldExecuteOnResume=true;
                        if(state == Constants.STATUS_NOT_READY){
                            state=Constants.STATUS_READY;
                        }
                        enableAllButtons();
                        break;
                    case Constants.ENABLE_BT:
                        enableBt();
                        break;
                    case Constants.CONNECTION_LOST:
                        connectionLost();
                        break;
                    case Constants.MESSAGE_READ:
                        String msg = (String) inputMessage.obj;
                        if(msg.equals("CONFOK")){
                            Log.v(TAG, "Konfigurazioa ondo ezarri da");
                        }else if(msg.equals("CONFKO")){
                            Log.e(TAG, "Konfigurazioa ezin izan da ezarri");
                        }else if(msg.equals("DENDR")){
                            Log.v(TAG, "Dolly-a bidearen eskuineko mugara iritsi da");
                            state=Constants.STATUS_READY;
                            enableAllButtons();
                        }else if(msg.equals("DENDL")){
                            Log.v(TAG, "Dolly-a bidearen ezkerreko mugara iritsi da");
                            state=Constants.STATUS_READY;
                            enableAllButtons();
                        }else if(msg.equals("FAIL")){
                            Log.v(TAG, "Time-lapseak huts egin du");
                            Toast.makeText(ctx, ctx.getString(R.string.toast_timelapse_fail), Toast.LENGTH_SHORT).show();
                            state=Constants.STATUS_READY;
                            enableAllButtons();
                        }else if(msg.equals("FIN_")){
                            Log.v(TAG, "Time-lapsea amaitu da");
                            Toast.makeText(ctx, ctx.getString(R.string.toast_timelapse_end), Toast.LENGTH_SHORT).show();
                            state=Constants.STATUS_READY;
                            enableAllButtons();
                        }
                        Log.e(TAG, "Iritsitako komandoa ez da ezagutzen");
                        Log.e(TAG, "Jasotako mezua: " + msg);
                        break;
                    default:
                        break;
                }
            }

        };
    }


    /**
     * Bluetooth aktibatua ez badago aktibatzeko eskaria egin
     * Aktibatu badago, bilaketa berrabiariazi
     */
    public void checkConnection(){
        Log.v(TAG, "Checking connection");
        if (!btAdapter.isEnabled()){
            enableBt();
        }else{
            btAdapter.cancelDiscovery();
            btAdapter.startDiscovery();
        }
    }

    /**
     * Bluetooth-a aktibatzeko eskaera egin
     */
    private static final int REQUEST_ENABLE_BT = 1;
    public void enableBt() {
        Intent enableBtIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
        startActivityForResult(enableBtIntent, REQUEST_ENABLE_BT);
    }

    /**
     * Bluetootha aktibatzeko eskariaren erantzuna tratatu:
     * Ez bada aktibatu, mezua erakutsi
     * Parekatu bada, dispositiboaren bilaketa hasi
     */
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);
        if(requestCode == REQUEST_ENABLE_BT) {
            if (resultCode == RESULT_CANCELED) {
                Toast.makeText(getApplicationContext(), getString(R.string.toast_bt_must_be_activated), Toast.LENGTH_LONG).show();
            } else if (resultCode == RESULT_OK) {
                checkConnection();
            }
        }
    }

    private void connectionLost(){
        Log.e(TAG, "Konexioa galdu da");
        Toast.makeText(ctx, ctx.getString(R.string.toast_connection_lost), Toast.LENGTH_SHORT).show();
        disableAllButtons();
    }

    private void disableAllButtons(){
        findViewById(R.id.btn_aStartStop).setEnabled(false);
        findViewById(R.id.btn_aAllLeft).setEnabled(false);
        findViewById(R.id.btn_aAllRight).setEnabled(false);
        findViewById(R.id.btn_mRigth).setEnabled(false);
        findViewById(R.id.btn_mLeft).setEnabled(false);
        findViewById(R.id.btn_mAllLeft).setEnabled(false);
        findViewById(R.id.btn_mAllRight).setEnabled(false);
        findViewById(R.id.btn_mStop).setEnabled(false);
        findViewById(R.id.btn_mShot).setEnabled(false);
    }

    private void enableAllButtons(){
        if(state == Constants.STATUS_TIMELAPSE_IN_PROGRESS){
            Button strStpBtn = (Button)findViewById(R.id.btn_aStartStop);
            strStpBtn.setEnabled(true);
            strStpBtn.setText(getString(R.string.button_stop));
            findViewById(R.id.btn_mStop).setEnabled(true);
        }else{
            Button strStpBtn = (Button)findViewById(R.id.btn_aStartStop);
            strStpBtn.setEnabled(true);
            strStpBtn.setText(getString(R.string.button_start));
            findViewById(R.id.btn_aAllLeft).setEnabled(true);
            findViewById(R.id.btn_aAllRight).setEnabled(true);
            findViewById(R.id.btn_mRigth).setEnabled(true);
            findViewById(R.id.btn_mLeft).setEnabled(true);
            findViewById(R.id.btn_mAllLeft).setEnabled(true);
            findViewById(R.id.btn_mAllRight).setEnabled(true);
            findViewById(R.id.btn_mStop).setEnabled(true);
            findViewById(R.id.btn_mShot).setEnabled(true);
        }
    }

    private int getLengthInSteps(int cm){
        double steps =  cm*36.326923;
        return (int) steps;
    }

    /**
     * Time-lapsea hasi baino lehen konprobaketa batzuk egingo dira
     * @param data
     */
    private void startTimeLapse(final String data){

        /** Argazki kopurua mugimendua baina handiagoa bada ezin da desplazamendu bat sortu */
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(ctx);
        int length_cm = Integer.valueOf(preferences.getString(ctx.getString(R.string.pref_lenght_key), "104"));
        int length_steps = getLengthInSteps(length_cm);
        int numOfPhototos_ = Integer.valueOf(data.substring(0, 5));
        CheckBox chBx_mov = (CheckBox) findViewById(R.id.chbx_mov);
        if(numOfPhototos_ == 0){
            Toast.makeText(ctx, ctx.getString(R.string.toast_selected_values_wrong), Toast.LENGTH_SHORT).show();
        }else if(((length_steps/numOfPhototos_) < 1) && chBx_mov.isChecked()){
            new AlertDialog.Builder(this)
                    .setTitle(ctx.getString(R.string.alertDialog_alert))
                    .setMessage(ctx.getString(R.string.alertDialog_too_much_photos))
                            .setIcon(android.R.drawable.ic_dialog_alert)
                            .setPositiveButton(ctx.getString(R.string.alertDialog_start), new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog, int whichButton) {
                                    Toast.makeText(MainActivity.this, ctx.getString(R.string.toast_timelapse_starting), Toast.LENGTH_SHORT).show();
                                    CheckBox chBx_mov = (CheckBox) findViewById(R.id.chbx_mov);
                                    chBx_mov.setChecked(false);
                                    findViewById(R.id.rb_leftToRight).setEnabled(false);
                                    findViewById(R.id.rb_rightToLeft).setEnabled(false);
                                    connection.write("STRT" + data + "\n");
                                    timelapseInProgress();
                                    state = Constants.STATUS_TIMELAPSE_IN_PROGRESS;
                                }
                            })
                            .setNegativeButton(ctx.getString(R.string.alertDialog_cancel), new DialogInterface.OnClickListener() {
                                public void onClick(DialogInterface dialog, int whichButton) {
                                    // Ezezkoa erantzuten bada
                                }
                            }).show();
        }else{
            connection.write("STRT" + data + "\n");
            timelapseInProgress();
            state=Constants.STATUS_TIMELAPSE_IN_PROGRESS;
        }


    }

    private void timelapseInProgress(){
        setStartStop_Stop();
        findViewById(R.id.btn_aAllLeft).setEnabled(false);
        findViewById(R.id.btn_aAllRight).setEnabled(false);
        findViewById(R.id.btn_mRigth).setEnabled(false);
        findViewById(R.id.btn_mLeft).setEnabled(false);
        findViewById(R.id.btn_mAllLeft).setEnabled(false);
        findViewById(R.id.btn_mAllRight).setEnabled(false);
        findViewById(R.id.btn_mShot).setEnabled(false);
    }

    public void setStartStop_Stop(){
        Button strtStp = (Button) findViewById(R.id.btn_aStartStop);
        strtStp.setText(getString(R.string.button_stop));
    }

    public void setStartStop_Start(){
        Button strtStp = (Button) findViewById(R.id.btn_aStartStop);
        strtStp.setText(getString(R.string.button_start));
    }

    private void showTimelapseStopDialog(){
        new AlertDialog.Builder(this)
                .setTitle(ctx.getString(R.string.alertDialog_alert))
                        .setMessage(ctx.getString(R.string.alertDialog_cancel_timeLapse))
                        .setIcon(android.R.drawable.ic_dialog_alert)
                        .setPositiveButton(ctx.getString(R.string.alertDialog_yes), new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int whichButton) {
                                Toast.makeText(MainActivity.this, "Time-lapsea gelditzen", Toast.LENGTH_SHORT).show();
                                connection.write("STOP\n");
                                state = Constants.STATUS_READY;
                                enableAllButtons();
                            }
                        })
                        .setNegativeButton(ctx.getString(R.string.alertDialog_no), new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int whichButton) {
                                //Toast.makeText(MainActivity.this, "Nooo!!!", Toast.LENGTH_SHORT).show();
                            }
                        }).show();
    }

    private void sendConfig(){
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(ctx);
        String cameraModel = preferences.getString(ctx.getString(R.string.pref_camera_key), "NI");
        String lenght_in_cm = preferences.getString(ctx.getString(R.string.pref_lenght_key), "104");

        connection.write("CONF" + cameraModel + getLengthInSteps(Integer.valueOf(lenght_in_cm)) + "\n");
    }

    /**
     * ACTIVITYaren BIZI-ZIKLOA
     * Aktibity-a zenbait egoeratatik igarotzen da
     * Egoera horiek detektatu eta behar den kasuan akzioak egin
     */

    @Override
    public void onResume() {
        super.onResume();

        if(shouldExecuteOnResume){
            // Your onResume Code Here
            sendConfig();
        }

        //Hizkuntza aldaketa detektatu
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(ctx);
        String lang = preferences.getString(ctx.getString(R.string.pref_language_key), "en");
        if(!lang.equals(prevLang)){
            myLocale = new Locale(lang);
            Resources res = getResources();
            DisplayMetrics dm = res.getDisplayMetrics();
            Configuration conf = res.getConfiguration();
            conf.locale = myLocale;
            res.updateConfiguration(conf, dm);
            Intent refresh = new Intent(this, MainActivity.class);
            startActivity(refresh);
            finish();
        }
       // Toast.makeText(this, "Resume...", Toast.LENGTH_SHORT).show();
    }
    @Override
    protected void onRestart(){
        super.onRestart();
        //Toast.makeText(this, "Restart...", Toast.LENGTH_SHORT).show();
    }
    @Override
    protected void onStart(){
        super.onStart();
        //Toast.makeText(this, "Start...", Toast.LENGTH_SHORT).show();
    }
    @Override
    protected void onStop(){
        super.onStop();
       // Toast.makeText(this, "Stop...", Toast.LENGTH_SHORT).show();
    }
    @Override
    protected void onPause(){
        super.onPause();
        //Toast.makeText(this, "Pause...", Toast.LENGTH_SHORT).show();
    }

    /**
     * Menuaren botoia sakatzean aukerak erakustsi
     * @param menu
     * @return
     */

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_remote_dolly, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            Log.v(TAG, "Menu>Setings");
            Intent intent = new Intent(this, SettingsActivity.class);
            startActivity(intent);
        } else if (id == R.id.action_connect) {
            Log.v(TAG, "Menu>Connect");
            checkConnection();
        }

        return super.onOptionsItemSelected(item);
    }


    /**
     * UI kontrolatzeko metodoak
     *
     * @param tab
     * @param fragmentTransaction
     */
    @Override
    public void onTabSelected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
        // When the given tab is selected, switch to the corresponding page in
        // the ViewPager.
        mViewPager.setCurrentItem(tab.getPosition());
    }

    @Override
    public void onTabUnselected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
    }

    @Override
    public void onTabReselected(ActionBar.Tab tab, FragmentTransaction fragmentTransaction) {
    }


    /**
     * A {@link FragmentPagerAdapter} that returns a fragment corresponding to
     * one of the sections/tabs/pages.
     */
    public class SectionsPagerAdapter extends FragmentPagerAdapter {

        public SectionsPagerAdapter(FragmentManager fm) {
            super(fm);
        }

        @Override
        public Fragment getItem(int position) {
            if (position == 0) {
                return new TimelapseFragment();
            } else if (position == 1) {
                return new ManualFragment();
            }else {
                return null;
            }

        }

        @Override
        public int getCount() {
            //Return num of pages
            return 2;
        }

        @Override
        public CharSequence getPageTitle(int position) {
            Locale l = Locale.getDefault();
            switch (position) {
                case 0:
                    return getString(R.string.title_section1).toUpperCase(l);
                case 1:
                    return getString(R.string.title_section2).toUpperCase(l);
            }
            return null;
        }
    }



}
