package gap.enekow.dlappse;

/**
 * Created by enekow on 27/05/2015.
 */

import android.app.Activity;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.preference.PreferenceManager;
import android.support.v4.app.Fragment;
import android.util.Log;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.NumberPicker;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;

public class TimelapseFragment extends Fragment implements NumberPicker.OnValueChangeListener {

    String TAG = "GAKOA";

    TimelapseFragmentListener mCallback;

    // Container Activity must implement this interface
    public interface TimelapseFragmentListener {
        void sendCommand(int cmd, String data);
        void setStartStop_Start();
        void setStartStop_Stop();
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);

        // This makes sure that the container activity has implemented
        // the callback interface. If not, it throws an exception
        try {
            mCallback = (TimelapseFragmentListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(activity.toString()
                    + " must implement OnHeadlineSelectedListener");
        }
    }


    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {

        View rootView = inflater.inflate(R.layout.fragment_timelapse, container, false);

        RadioButton rb_aceleration = (RadioButton) rootView.findViewById(R.id.rb_aceleration);
        RadioButton rb_frameInterval = (RadioButton) rootView.findViewById(R.id.rb_frameInterval);
        Spinner sp_aceleration = (Spinner) rootView.findViewById(R.id.spinner_aceleration);
        Spinner sp_frameInt = (Spinner) rootView.findViewById(R.id.spinner_frameInterval);
        CheckBox chBx_mov = (CheckBox) rootView.findViewById(R.id.chbx_mov);

        NumberPicker np_h = (NumberPicker) rootView.findViewById(R.id.np_hours);
        NumberPicker np_m = (NumberPicker) rootView.findViewById(R.id.np_minutes);

        //Hasierako konfigurazioa
        sp_aceleration.setEnabled(false);
        np_h.setMaxValue(24);
        np_h.setMinValue(0);
        np_m.setMaxValue(59);
        np_m.setMinValue(0);
        np_m.setValue(0);
        np_m.setFormatter(new NumberPicker.Formatter() {
            @Override
            public String format(int i) {
                return String.format("%02d", i);
            }
        });

        //Eventoak
        rb_aceleration.setOnClickListener(new View.OnClickListener() {
            // @Override
            public void onClick(View v) {
                acelerationSelected();
            }
        });

        rb_frameInterval.setOnClickListener(new View.OnClickListener() {
            // @Override
            public void onClick(View v) {
                frameIntervalSelected();
            }
        });

        sp_aceleration.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                refreshSummary();
            }

            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        sp_frameInt.setOnItemSelectedListener(new AdapterView.OnItemSelectedListener() {
            @Override
            public void onItemSelected(AdapterView<?> parent, View view, int position, long id) {
                refreshSummary();
            }
            @Override
            public void onNothingSelected(AdapterView<?> parent) {

            }
        });

        chBx_mov.setOnCheckedChangeListener(new CompoundButton.OnCheckedChangeListener() {
            public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                RadioButton rb_lTr = (RadioButton) getActivity().findViewById(R.id.rb_leftToRight);
                RadioButton rb_rTl = (RadioButton) getActivity().findViewById(R.id.rb_rightToLeft);
                if (isChecked) {
                    rb_lTr.setEnabled(true);
                    rb_rTl.setEnabled(true);
                } else {
                    rb_lTr.setEnabled(false);
                    rb_rTl.setEnabled(false);
                }
            }
        });

        //Denbora aukeratzean listenerrari deitu
        np_h.setOnValueChangedListener(this);
        np_m.setOnValueChangedListener(this);

        //Start/Stop sakatzean
        final Button btnStartStop = (Button) rootView.findViewById(R.id.btn_aStartStop);
        btnStartStop.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.v(TAG, "Start/Stop sakatu da");
                startStopTimelapse();
            }
        });

        Button btnAllLeft = (Button) rootView.findViewById(R.id.btn_aAllLeft);
        btnAllLeft.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.v(TAG, "AllLeft sakatu da");
                mCallback.setStartStop_Stop();
                mCallback.sendCommand(Constants.GO_LEFT, "7");
            }
        });
        Button btnAllRight = (Button) rootView.findViewById(R.id.btn_aAllRight);
        btnAllRight.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Log.v(TAG, "AllRight sakatu da");
                mCallback.setStartStop_Stop();
                mCallback.sendCommand(Constants.GO_RIGHT, "7");
            }
        });

        //Hasieran botoiak desaktibatuak
        rootView.findViewById(R.id.btn_aStartStop).setEnabled(false);
        rootView.findViewById(R.id.btn_aAllRight).setEnabled(false);
        rootView.findViewById(R.id.btn_aAllLeft).setEnabled(false);

        return rootView;
    }

    public void onResume(){
        super.onResume();
        refreshSummary();
    }

    private void startStopTimelapse(){
        int numPhotos;
        double frameInterval;

        frameInterval = getFrameInterval();
        numPhotos = getNumberOfPhotos();

        //Argazki kopuruarekin 5 karaktereko String-a sortu, hasieran 0-ak erantsiz
        String numberOfPhotosString = String.format("%05d", numPhotos);
        Log.v(TAG, "Argazki kopurua: " + numberOfPhotosString);
        //Argazki tartearen balioekin String-a lortu
        String frameIntervalString = getFrameIntervalString(frameInterval);
        String dir = getDirection();

        //Klase nagusiari mezua bidaltzeko eskaera egin
        String timlapseData = numberOfPhotosString + frameIntervalString + dir;
        mCallback.sendCommand(Constants.START_STOP_TIMELAPSE, timlapseData);
    }

    private String getDirection(){
        if(isMoveChecked()){
            RadioButton rdBtnLR = (RadioButton) getActivity().findViewById(R.id.rb_leftToRight);
            if (rdBtnLR.isChecked()){
                return "LR";
            } else{
                return "RL";
            }
        }else{
            return "NO";
        }
    }

    private boolean isMoveChecked(){
        CheckBox moveCheck = (CheckBox) getActivity().findViewById(R.id.chbx_mov);
        if (moveCheck.isChecked()){
            return true;
        }else{
            return false;
        }
    }

     /**
     * Frame interval eman eta 5 karaktereko string-a itzultzen du
     * Balioa 2 hamarrenekotara borobiltzen da (ad.: 10.33)
     */
    private String getFrameIntervalString(double frmInt){
        String frameIntervalString = String.format("%.2f", frmInt);
        frameIntervalString = frameIntervalString.replace(',','.');

        if (frameIntervalString.length()<5){
            return "0" + frameIntervalString;
        }else{
            return frameIntervalString;
        }
    }

    //Iraupena aukeratzean
    public void onValueChange(NumberPicker picker, int oldVal, int newVal) {
        refreshSummary();
    }

    /**
     * Funtzio laguntzaileak
     */
    private void refreshSummary(){
        TextView text = (TextView) getActivity().findViewById(R.id.txt_summary);
        text.setText(getActivity().getString(R.string.txt_video_duration) + " \n" + String.valueOf(getMinSecResultDuration()) +
                " (" + String.valueOf(getNumberOfPhotos()) + " " + getActivity().getString(R.string.txt_photos) + ")");
    }

    /** Emaitzak izango duen iraupena minutu eta segundotan 0min 00seg */
    private String getMinSecResultDuration(){
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(getActivity());
        int fps = Integer.valueOf(preferences.getString(getActivity().getString(R.string.pref_fps_key), "25"));

        int resDurationInSec = (getNumberOfPhotos() / fps);

        int min = (resDurationInSec / 60);
        int sec = (resDurationInSec - min*60);

        String minStr = String.valueOf(min);
        String secStr = String.format("%02d", sec);

        return minStr + getActivity().getString(R.string.txt_min) + " " + secStr + getActivity().getString(R.string.txt_sec);
    }


    /** Kapturak iraungo duen denbora erreala segundotan */
    private int getDurationInSeconds(){
        NumberPicker np_h = (NumberPicker) getActivity().findViewById(R.id.np_hours);
        NumberPicker np_m = (NumberPicker) getActivity().findViewById(R.id.np_minutes);
        int sec=0;
        sec = (sec + (60 * np_m.getValue()));
        sec = (sec + ((60 * 60) * np_h.getValue()));
        return sec;
    }

    private int getNumberOfPhotos(){
        return (int) (getDurationInSeconds()/getFrameInterval());
    }

    private double getFrameInterval(){
        SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(getActivity());
        int fps = Integer.valueOf(preferences.getString(getActivity().getString(R.string.pref_fps_key), "25"));

        if (isRdioBttnFrameSelected()) {
            Spinner spin2 = (Spinner) getActivity().findViewById(R.id.spinner_frameInterval);
            int spinner_pos = spin2.getSelectedItemPosition();
            String[] size_values = getResources().getStringArray(R.array.frame_interval_values);
            return Double.valueOf(size_values[spinner_pos]);
        }else{
            int aceleration = getAceleration();
            return ((double)aceleration/(double)fps);
        }
    }

    private int getAceleration(){
        Spinner spin = (Spinner) getActivity().findViewById(R.id.spinner_aceleration);
        int spinner_pos = spin.getSelectedItemPosition();
        String[] size_values = getResources().getStringArray(R.array.capture_aceleration_values);
        int val = Integer.valueOf(size_values[spinner_pos]);
        return val;
    }

    private void acelerationSelected() {
        RadioButton rb2 = (RadioButton) getActivity().findViewById(R.id.rb_frameInterval);
        Spinner spin = (Spinner) getActivity().findViewById(R.id.spinner_aceleration);
        Spinner spin2 = (Spinner) getActivity().findViewById(R.id.spinner_frameInterval);

        rb2.setChecked(false);
        spin.setEnabled(true);
        spin2.setEnabled(false);
        refreshSummary();
    }

    private void frameIntervalSelected(){
        RadioButton rb1 = (RadioButton) getActivity().findViewById(R.id.rb_aceleration);
        Spinner spin = (Spinner) getActivity().findViewById(R.id.spinner_aceleration);
        Spinner spin2 = (Spinner) getActivity().findViewById(R.id.spinner_frameInterval);

        rb1.setChecked(false);
        spin.setEnabled(false);
        spin2.setEnabled(true);
        refreshSummary();
    }

    private boolean isRdioBttnFrameSelected(){
        RadioButton rb_interval = (RadioButton) getActivity().findViewById(R.id.rb_frameInterval);
        return rb_interval.isChecked();
    }

}
