
/**************************************************************************************
	
		Copyright 2015 Eneko Irazusta Aizpurua 

		Licensed under the Apache License, Version 2.0 Apache License;
		you may not use this file except in compliance with the License.
		You may obtain a copy of the License at

		    http://www.apache.org/licenses/LICENSE-2.0

		Unless required by applicable law or agreed to in writing, software
		distributed under the License is distributed on an "AS IS" BASIS,
		WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
		See the License for the specific language governing permissions and
		limitations under the License.

**************************************************************************************/

#include "Arduino.h"
#include "StepperMotor.h"

#define pwm_chA 3
#define pwm_chB 11
#define dir_chA 12
#define dir_chB 13
#define brake_chA 9
#define brake_chB 8
#define StepperMEMA17 200

int steepState = 1;

/* This constructor creates an instance with default configuration of Arduino Motor Shield Rev3 and MEMA17 Stepper */
StepperMotor::StepperMotor(){
	_pwm_chA = pwm_chA;
	_pwm_chB = pwm_chB;
	_dir_chA = dir_chA;
	_dir_chB = dir_chB;
	_brake_chA = brake_chA;
	_brake_chB = brake_chB;
	_steps = StepperMEMA17;
	_one_rpm_delay = (int)(60000/_steps);

	pinMode(_pwm_chA, OUTPUT);
	pinMode(_pwm_chB, OUTPUT);
	pinMode(_brake_chA, OUTPUT);
	pinMode(_brake_chB, OUTPUT);
	pinMode(_dir_chA, OUTPUT);
	pinMode(_dir_chB, OUTPUT);
}

/* This constructor creates an instance with the given configuration of shield and steper pins */
StepperMotor::StepperMotor(int pin_pwm_chA, int pin_pwm_chB, int pin_dir_chA, int pin_dir_chB, int pin_brake_chA, int pin_brake_chB, int num_steps){
	_pwm_chA = pin_pwm_chA;
	_pwm_chB = pin_pwm_chB;
	_dir_chA = pin_dir_chA;
	_dir_chB = pin_dir_chB;
	_brake_chA = pin_brake_chA;
	_brake_chB = pin_brake_chB;
	_steps = num_steps;

	pinMode(_pwm_chA, OUTPUT);
	pinMode(_pwm_chB, OUTPUT);
	pinMode(_brake_chA, OUTPUT);
	pinMode(_brake_chB, OUTPUT);
}

/* Use this method to move the steper at a given speed */
void StepperMotor::speed(int rpm, int dir){
	if(!dir){
		goDir0((int) (_one_rpm_delay/rpm));
	}else{
		goDir1((int) (_one_rpm_delay/rpm));
	}
}

/* Use this method to move the steper one step */
int StepperMotor::step(int dir){
	if(!dir){
		goDir0(14);		
	}else{
		goDir1(14);
	}
	return 14;
}

/* Use this method to move the stepper the given number of steps */
/* Also you can select the direction of the rotation */
/* It returns the time taken completing the task (in miliseconds) */
/* ATENTION: these method blocks the main thread until all the steps are done */
int StepperMotor::steps(int steeps, int dir){
	int miliseconds=0;
	if(!dir){
		for(int i=0; i<steeps; i++){
			goDir0(14);
			miliseconds = miliseconds+14;
		}
	}else{
		for(int i=0; i<steeps; i++){
			goDir1(14);
			miliseconds= miliseconds+14;
		}
	}
	return miliseconds;
}

/* Use this method to move the stepper the given number of steps */
/* Also you can select the direction of the rotation and the rpm */
/* ATENTION: these method blocks the main thread until all the steps are done */
void StepperMotor::steps(int steeps, int rpm, int dir){
	if(!dir){
		for(int i=0; i<steeps; i++){
			goDir0((int) (_one_rpm_delay/rpm));
		}
	}else{
		for(int i=0; i<steeps; i++){
			goDir1((int) (_one_rpm_delay/rpm));
		}
	}
}

/* Use this method to stop the stepper */
void StepperMotor::stop(){
	digitalWrite(_brake_chA, HIGH);
	digitalWrite(_brake_chB, HIGH);
}

/*-----------------------------------------------------------------------------*/
/* The next two methods are internal functions used by the other methods. 
DO NOT use them directly unless you know what are you doing. */
void StepperMotor::goDir0(int delaylegnth){	
	switch(steepState){
		case 1:
			digitalWrite(_brake_chA, LOW);  //ENABLE CH A
			digitalWrite(_brake_chB, HIGH); //DISABLE CH B
			digitalWrite(_dir_chA, HIGH);   //Sets direction of CH A
			digitalWrite(_pwm_chA, HIGH);   //Moves CH A
			delay(delaylegnth);
			steepState = 2;
			break;

		case 2:
			digitalWrite(_brake_chA, HIGH);  //DISABLE CH A
			digitalWrite(_brake_chB, LOW); //ENABLE CH B
			digitalWrite(_dir_chB, LOW);   //Sets direction of CH B
			digitalWrite(_pwm_chB, HIGH);   //Moves CH B
			delay(delaylegnth);
			steepState = 3;
			break;

		case 3:
			digitalWrite(_brake_chA, LOW);  //ENABLE CH A
			digitalWrite(_brake_chB, HIGH); //DISABLE CH B
			digitalWrite(_dir_chA, LOW);   //Sets direction of CH A
			digitalWrite(_pwm_chA, HIGH);   //Moves CH A
			delay(delaylegnth);
			steepState = 4;
			break;

		case 4:
			digitalWrite(_brake_chA, HIGH);  //DISABLE CH A
			digitalWrite(_brake_chB, LOW); //ENABLE CH B
			digitalWrite(_dir_chB, HIGH);   //Sets direction of CH B
			digitalWrite(_pwm_chB, HIGH);   //Moves CH B
			delay(delaylegnth);
			steepState = 1;
			break;
	}
}

void StepperMotor::goDir1(int delaylegnth){
	switch(steepState){
		case 3:
			digitalWrite(_brake_chA, LOW);  //ENABLE CH A		
			digitalWrite(_brake_chB, HIGH); //DISABLE CH B		
			digitalWrite(_dir_chA, HIGH);   //Sets direction of CH A		
			digitalWrite(_pwm_chA, HIGH);   //Moves CH A		
			delay(delaylegnth);
			steepState = 4;
			break;

		case 4:
			digitalWrite(_brake_chA, HIGH);  //DISABLE CH A		
			digitalWrite(_brake_chB, LOW); //ENABLE CH B		
			digitalWrite(_dir_chB, HIGH);   //Sets direction of CH B		
			digitalWrite(_pwm_chB, HIGH);   //Moves CH B		
			delay(delaylegnth);
			steepState = 1;
			break;

		case 1:
			digitalWrite(_brake_chA, LOW);  //ENABLE CH A
			digitalWrite(_brake_chB, HIGH); //DISABLE CH B
			digitalWrite(_dir_chA, LOW);   //Sets direction of CH A
			digitalWrite(_pwm_chA, HIGH);   //Moves CH A
			delay(delaylegnth);
			steepState = 2;
			break;

		case 2:
			digitalWrite(_brake_chA, HIGH);  //DISABLE CH A
			digitalWrite(_brake_chB, LOW); //ENABLE CH B
			digitalWrite(_dir_chB, LOW);   //Sets direction of CH B
			digitalWrite(_pwm_chB, HIGH);   //Moves CH B
			delay(delaylegnth);
			steepState = 3;
			break;
	}
}