#include <multiCameraIrControl.h>
#include <StepperMotor.h>
#include <SoftwareSerial.h>

/* BLUETOOTH SERIALa KONFIGURATU */
#define RxD 4
#define TxD 5
SoftwareSerial BTSerial(TxD, RxD);
/* PAUSOKAKO MOTORRA KONFIGURATU */
StepperMotor motorra;
/* KAMERA MODELO EZBEDINAK LORTU */
Nikon nikon(7);
Canon canon(7);
Olympus olympus(7);
Sony sony(7);
Minolta minolta(7);
Pentax pentax(7);

int ezkerBotoiaPin = 6;
int eskubiBotoiaPin = 2;
int ezkerBotoia = LOW;
int eskubiBotoia = LOW;


/* EGOERAK */
#define GELDIK 0
#define EZKERRERA 1
#define ESKUBIRA 2
#define TIME_LAPSE 3


/* MOTORRA ETA MUGIMENDUAREN ALDAGAIAK */
int rpm = 0;
int steps = 0;
int miliseconds = 0;
int desplazamendua = 0;
int abiadura = 5;
int movDir;
int egindakoPausoKop=0;
/*  */
String inputString = "";
int numPhotos;
int frameInterval;
int timelapse_egoera = 0;
int bide_luzera = 3778;
String kamera_modeloa = "NI";

int egoera = 0;
int timeEgoera = 0;

void setup() {

  pinMode(ezkerBotoiaPin, INPUT);
  pinMode(eskubiBotoiaPin, INPUT);
  
  Serial.begin(9600);
  BTSerial.begin(9600);
  BTSerial.flush();
  delay(500);
  Serial.print("Prest\n");
}

void loop() {
  /* Bluetooth bidez zerbait jaso den aztertu */
  while (BTSerial.available()) {
    char inChar = (char)BTSerial.read();
    inputString += inChar;
    if (inChar == '\n') {
      handleMessage(inputString);
      inputString = "";
    }
  }
  
  /* Jasotako abiadura faktorearen baitan balioa kalkulatu */
  rpm = 10 + (5*abiadura);
  
  /* Irakurri alboetako etengailuak */
  eskubiBotoia = digitalRead(eskubiBotoiaPin);
  ezkerBotoia = digitalRead(ezkerBotoiaPin);
  if(eskubiBotoia == HIGH){
    Serial.print("Eskubiko etengailua zapaldua\n");  
     if (egoera == 7) {
        Serial.print("Steps: " + String(steps) + "\n");
        steps = 0;
        egoera=GELDIK;
    }
  }
  if(ezkerBotoia == HIGH){
    Serial.print("Ezkerreko etengailua zapaldua\n");  
     if (egoera == 7) {
        Serial.print("Steps: " + String(steps) + "\n");
        steps = 0;
        egoera=GELDIK;
    }
  }  
  
  /* Egoera tratatu */
  switch (egoera) {
    
    case GELDIK:
      motorra.stop();
      break;
      
    case EZKERRERA:
        if(ezkerBotoia == LOW){
          motorra.speed(rpm, 0);
        }else{
          motorra.stop();
          BTSerial.write("DENDL\n");
          egoera=GELDIK;
        }
        break;
        
    case ESKUBIRA:
      if(eskubiBotoia == LOW){
        motorra.speed(rpm, 1);
      }else{
          motorra.stop();
          BTSerial.write("DENDR\n");
          egoera=GELDIK;
      }
      break;
      
    case TIME_LAPSE:
        if(((ezkerBotoia == HIGH) && (movDir==0)) || ((eskubiBotoia == HIGH)&&(movDir==1))){
            Serial.print("Time-lapsea eten egin da\n");
            BTSerial.print("FIN_\n");
            egoera=GELDIK;
            break;
        }
        switch (timelapse_egoera) {
            case 0:
                desplazamendua = (bide_luzera / numPhotos)/2;
                if(egindakoPausoKop <= desplazamendua){
                    miliseconds = miliseconds + motorra.step(movDir);
                    egindakoPausoKop++;
                    break;
                }else{
                    Serial.print("Desplazamendua" + String(desplazamendua) + "\n");
                    Serial.print("Miliseconds" + String(miliseconds) + "\n");
                    egindakoPausoKop=0;
                    delay((frameInterval/2) - miliseconds);
                    timelapse_egoera = 1;
                    break;
                }
                break;
            case 1:
                argazkiaAtera();
                delay((frameInterval/2) - miliseconds);
                miliseconds=0;
                timelapse_egoera = 2;
                break;
            case 2:
                desplazamendua = (bide_luzera / numPhotos)/2;
                if(egindakoPausoKop <= desplazamendua){
                    miliseconds = miliseconds + motorra.step(movDir);
                    egindakoPausoKop++;
                }else{
                    Serial.print("Desplazamendua" + String(desplazamendua) + "\n");
                    Serial.print("Miliseconds" + String(miliseconds) + "\n");
                    egindakoPausoKop=0;
                    miliseconds=0;
                    timelapse_egoera = 0;
                }
                break;
            case 3:
                argazkiaAtera();
                delay(frameInterval);
                break;
            }
        break;
    
    case 7:
      steps++;
      motorra.speed(rpm, 1);
      break;
  } //switch egoera amaiera
} // end loop

#define STOP_ 0
#define START_ 1
#define LEFT_ 2
#define RIGHT_ 3
#define SHOT_ 4
#define CONF_ 5

void handleMessage(String msg) {
  String command = msg.substring(0, 4);
  Serial.print("Komandoa: " + command + "\n");
  Serial.print("Mezu osoa: " + msg);
  switch (komadoaLortu(command)) {
    case STOP_:
      Serial.print("Gelditu egingo da\n");
      egoera = GELDIK;
      break;
    case START_:
      {        
        String numPhotosString = msg.substring(4, 9);
        String frameIntervalString = msg.substring(9, 14);
        String movDirString = msg.substring(14, 16);
        
        Serial.print("Start with these values:\n");
        Serial.print("Photos: " + numPhotosString + "\n");
        Serial.print("Interval: " + frameIntervalString + "\n");
        Serial.print("Direction: " + movDirString + "\n");
        
        numPhotos = numPhotosString.toInt();
        double frameInterval_double = frameIntervalString.toFloat();
        frameInterval_double = frameInterval_double * 1000;
        frameInterval = (int) frameInterval_double;
        
        if(movDirString=="NO"){
            //Ez mugitu
            timelapse_egoera = 3;
        }else if(movDirString == "LR"){
          movDir = 1;
          timelapse_egoera = 0;
          //Alde horretara joan daiteken begiratu
          if(eskubiBotoia==HIGH){            
            Serial.print("Ezin da timelapsea eskuinera egin\n");
            BTSerial.print("FAIL\n");
            egoera=GELDIK;
            break;
          }
        }else if(movDirString == "RL"){
          movDir = 0;
          timelapse_egoera = 0;
          if(ezkerBotoia==HIGH){
            Serial.write("Ezin da timelapsea ezkerrera egin\n");
            BTSerial.print("FAIL\n");
            egoera=GELDIK;
            break;
          }
        }
       
        egoera = TIME_LAPSE;

        break;
      }
    case LEFT_:
      Serial.print("Ezkerrera joango da\n");
      abiadura = msg.substring(4, 5).toInt();
      abiadura++;
      //Ezkerreko etengaiula konprobatu
      if(ezkerBotoia==HIGH){
            egoera=GELDIK;
            Serial.write("Ezin da joan ezkerrerago\n");
            break;
      }
      egoera = EZKERRERA;
      break;
    case RIGHT_:
      Serial.print("Eskubira joango da\n");
      abiadura = msg.substring(4, 5).toInt();
      abiadura++;
      //Ezkerreko etengaiula konprobatu
      if(eskubiBotoia==HIGH){
            egoera=GELDIK;
            Serial.write("Ezin da joan eskubirago\n");
            break;
      }
      egoera = ESKUBIRA;
      break;
    case SHOT_:
      Serial.print("Argazkia aterako du\n");
      argazkiaAtera();
      break;
    case CONF_:
      String kam_modeloa = msg.substring(4, 6);
      int luzera = msg.substring(6, 10).toInt();
      Serial.print("Kamera: " + kam_modeloa + "\n");
      Serial.print("Luzera: " + String(luzera) + "\n");
      konfigurazioa_ezarri(kam_modeloa, luzera);
      break;
  }
}// end mezuaTratatu()

void argazkiaAtera(){
  if(kamera_modeloa == "NI"){
    nikon.shutterNow();
  }else if(kamera_modeloa == "CA"){
    canon.shutterNow();
  }else if(kamera_modeloa == "OL"){
    olympus.shutterNow();
  }else if(kamera_modeloa == "PE"){
    pentax.shutterNow();
  }else if(kamera_modeloa == "SO"){
    sony.shutterNow();
  }else if(kamera_modeloa == "MI"){
    minolta.shutterNow();
  }
}

void konfigurazioa_ezarri(String kam, int luz){
  bide_luzera = luz;
  kamera_modeloa = kam;
  Serial.print("Konfigurazio berria ezarri da\n");
  BTSerial.print("CONFOK\n");
}

int komadoaLortu(String cmd) {
  if (cmd == "STOP") {
    return 0;
  } else if (cmd == "STRT") {
    return 1;
  } else if (cmd == "LEFT") {
    return 2;
  } else if (cmd == "RIGH") {
    return 3;
  } else if (cmd == "SHOT") {
    return 4;
  }else if (cmd == "CONF") {
    return 5;
  }else{
    return 0;
  }
}

/* DEBUG USB bidez */
void serialEvent() {
  while (Serial.available()) {
    // get the new byte:
    char inChar = (char)Serial.read();
    // add it to the inputString:
    inputString += inChar;
    // if the incoming character is a newline, set a flag
    // so the main loop can do something about it:
    if (inChar == '\n') {
      egoera = 7;
    }
  }
}
